/* (c) Copyright IBM Corp. 1996, 1997.  All rights reserved.      */
/*                                                                */
/* US Government Users Restricted Rights - Use, duplication or    */
/* disclosure restricted by GSA ADP Schedule Contract with IBM    */
/* Corp.                                                          */
/*                                                                */
/* DISCLAIMER OF WARRANTIES.  The following [enclosed] code is    */
/* sample code created by IBM Corporation.  This sample code is   */
/* not part of any standard or IBM product and is provided to you */
/* solely for the purpose of assisting you in the development of  */
/* your applications.  The code is provided "AS IS", without      */
/* warranty of any kind.  IBM shall not be liable for any damages */
/* arising out of your use of the sample code, even if they have  */
/* been advised of the possibility of such damages.               */

/*
**
**      env.h
**
**      This file contains definitions providing a language environment
**      that masks platform and compiler's local accent, and gives more
**      descriptive definitions.
**
**
*/

#ifndef H_ENV
#define H_ENV

#define TRUE    1
#define FALSE   0
typedef unsigned int    Boolean;

#define RC_OK           0                       /* return code, everything is fine */
#define RC_NOK          -1                      /* return code, something is wrong */

typedef unsigned char   Bit8;                   /* one octet */
typedef unsigned short  Bit16;                  /* two octets */
typedef unsigned long   Bit32;                  /* four octets */
typedef float           IEEEFloat;


typedef unsigned long   Counter;                /* statistics */
typedef unsigned long   Gauge;                  /* instrument */
typedef unsigned long   GaugeLimit;             /* upper bound of gauge */

typedef int             Limit;                  /* of index, max */


typedef unsigned long   TimeTick;               /* s, ms, us, or ns */

typedef Bit32           Bandwidth;              /* bits per second */
typedef Bit8            BWShare;                /* in percentage (%) */

typedef Bit8            State;                  /* of an entity */
typedef Bit8            Type;                   /* of an item */

/* Buffer anchor */
typedef struct {
    Bit32       size;           /* total usable size of buffer in bytes */
    Bit8        *pbuffer;       /* pointer to memory of buffer */
    Bit32       usedLen;        /* number of bytes in use */

} Buffer;


/* Timer definition */
typedef struct  {
    Bit8        type;                   /* of timer, thus interval */
    Bit8        numRetries;             /* so far */
    Bit8        maxRetries;             /* limit */

} Timer;

/* Timers, more than one may be running for a given state */
#define TIMER_NONE      0x00
#define TIMER_T1        0x01
#define TIMER_T2        0x02
#define TIMER_T3        0x04
#define TIMER_T4        0x08
#define TIMER_ALL       0xff


/* IP address */
typedef Bit32   IP4Address;     /* IPv4 address, host order */
typedef struct  {
    Bit8        byte[16];

} IP6Address;                   /* IPv6 address */

/* types used in the ISI RAPI header file */

typedef unsigned char   u_int8_t;                   /* Bit8 */
typedef unsigned short  u_int16_t;                  /* Bit16 */
typedef unsigned long   u_int32_t;                  /* Bit32 */
typedef signed char     int8_t;
typedef signed short    int16_t;
typedef signed long     int32_t;

#endif /* H_ENV */


/* End of env.h */
