/* (c) Copyright IBM Corp. 1996, 1997.  All rights reserved.      */
/*                                                                */
/* US Government Users Restricted Rights - Use, duplication or    */
/* disclosure restricted by GSA ADP Schedule Contract with IBM    */
/* Corp.                                                          */
/*                                                                */
/* DISCLAIMER OF WARRANTIES.  The following [enclosed] code is    */
/* sample code created by IBM Corporation.  This sample code is   */
/* not part of any standard or IBM product and is provided to you */
/* solely for the purpose of assisting you in the development of  */
/* your applications.  The code is provided "AS IS", without      */
/* warranty of any kind.  IBM shall not be liable for any damages */
/* arising out of your use of the sample code, even if they have  */
/* been advised of the possibility of such damages.               */

#ifndef H_RSVP_RAPI
#define H_RSVP_RAPI

#ifndef OS2
#include <sys/types.h>
#endif

#include <sys/socket.h> /* for sockaddr */
#include <netinet/in.h>  /* for sockaddr_in */

/* TEMPORARY! */
typedef float   float32_t;

/*****************************************************************************/

/*
 *   Filterspec Formats
 */

typedef enum {
        RAPI_FILTER_NONE  = 0,    /* Empty filter */
        RAPI_FILTERFORM_1 = 1,    /* Complex: sockaddr + (mask, value) */
        RAPI_FILTERFORM_2 = 2     /* Simple: Only sockaddr */
} rapi_filterform_t;

#define RAPI_MAXFILTER_T1       48

typedef struct {
        struct sockaddr sender;    /* or use another defined struct */
        u_int32_t       len;
        u_char          v[RAPI_MAXFILTER_T1];
        u_char          m[RAPI_MAXFILTER_T1];
} rapi_filter_t1_t;

typedef struct {
        struct sockaddr_in sender;  /* or use another defined struct */
} rapi_filter_t2_t;

/*
 *      RAPI Filterspec
 */
typedef struct {
        u_int32_t               len;    /* length of filter, in bytes */
        rapi_filterform_t       form;   /* filter type */
        union {
            rapi_filter_t1_t    t1;
            rapi_filter_t2_t    t2;
        }           filter;             /* variable length */
} rapi_filter_t;


/*****************************************************************************/

/*
 *      Int-serv Flowspec/Tspec
 */

#define TB_RATE_MAX             40e12      /* Bytes/sec */
#define TB_RATE_MIN             1          /* Bytes/sec */
#define TB_DEPTH_MAX            250e9      /* Bytes */
#define TB_DEPTH_MIN            1          /* Bytes */
#define TB_PEAK_MAX             0x7f800000 /* +ve infinity */

typedef struct {
        float32_t       IS_Tspec_r;             /* Token bucket rate (B/sec) */
        float32_t       IS_Tspec_b;             /* Token bucket depth (B) */
        float32_t       IS_Tspec_p;             /* Peak data rate (B/sec) */
        u_int32_t       IS_Tspec_m;             /* Min Policed Unit (B) */
        u_int32_t       IS_Tspec_M;             /* Max pkt size (B)     */
}   IS_Tspec_t;


typedef struct {
        float32_t       Guar_Rspec_R;            /* Guaranteed Rate (B/sec) */
        u_int32_t       Guar_Rspec_S;            /* Slack term (sec)    */
}   Guar_Rspec_t;


typedef struct {
        IS_Tspec_t     CL_Tspec;
}   CL_flowspec_t;

typedef struct {
        IS_Tspec_t       Guar_Tspec;
        Guar_Rspec_t     Guar_Rspec;
}   Guar_flowspec_t;


/*
 *      Legacy Flowspec
 */

typedef enum {
        QOS_TSPEC       = 0,
        QOS_CNTR_DELAY  = 1,
        QOS_PREDICTIVE  = 2,
        QOS_GUARANTEED  = 3
} qos_service_type;

typedef struct {
        qos_service_type  spec_type;            /* qos_service_type */
        struct {
                u_int32_t       spec_Tspec_b;   /* Token bucket depth (bits) */
                u_int32_t       spec_Tspec_r;   /* Token bucket avg rate (bps)*/
        }                 spec_Tspec;
        u_int32_t         spec_d;               /* Target Delay (ms) */
        u_int32_t         spec_sl;              /* Service Level */
}   qos_flowspec_t;


/*
 *      RAPI flowspec/Tspec formats
 */

typedef enum {
        RAPI_FORMAT_NONE       = 0,     /* empty flowspec/tspec        */
        RAPI_FORMAT_LEGACY     = 1,     /* qos: Legacy format          */
        RAPI_FORMAT_IS_CL      = 2,     /* Int-serv Controlled Load    */
        RAPI_FORMAT_IS_GUAR    = 3,     /* Int-serv Guaranteed         */
        RAPI_FORMAT_IS_GEN     = 4      /* Int-serv generic            */
} rapi_format_t;

/*
 *      RAPI Flowspec
 */

typedef struct {
        u_int32_t       len;
        rapi_format_t   form;
        union {
                qos_flowspec_t   qos;       /* Legacy format */
                CL_flowspec_t    IS_CL;     /* Int-serv Controlled Load   */
                Guar_flowspec_t  IS_Guar;   /* Int-serv Guaranteed format */
        }               specbody_u;
}   rapi_flowspec_t;


/*
 *      RAPI Tspec
 */

typedef struct {
        u_int32_t       len;
        rapi_format_t   form;
        union {
                qos_flowspec_t   qos;             /* Legacy format */
                IS_Tspec_t       IS_Tspec;        /* Int-serv (generic) */
        }               specbody_u;
}   rapi_tspec_t;


/*****************************************************************************/

/*
 *   Adspec Formats
 */

typedef struct {
        Boolean         servBreak;        /* break in service */
        u_int16_t       ISHops;           /* num Int-serv aware hops */
        float32_t       pathBW;           /* min path band width (bytes/sec) */
        u_int32_t       pathLatency;      /* min path latency (microseconds) */
        u_int16_t       composedMTU;      /* composed path MTU */
        u_int32_t       Ctot;             /* total rate dep. err (bytes) */
        u_int32_t       Dtot;             /* total rate indep. err (microseconds) */
        u_int32_t       Csum;             /* reshaped rate dep. err (bytes) */
        u_int32_t       Dsum;             /* reshaped rate indep. err (microseconds) */
} Guar_adspec_t;

typedef struct {
        Boolean         servBreak;        /* break in service */
        u_int16_t       ISHops;           /* num Int-serv aware hops */
        float32_t       pathBW;           /* min path band width (bytes/sec) */
        u_int32_t       pathLatency;      /* min path latency (microseconds) */
        u_int16_t       composedMTU;      /* composed path MTU */
} CL_adspec_t;

typedef struct {
        Boolean         CL_present;
        CL_adspec_t     CL_Adspec;
        Boolean         Guar_present;
        Guar_adspec_t   Guar_Adspec;
} IS_adspec_t;


/*
 *      RAPI Adspec formats
 */

typedef enum {
        RAPI_ADSPEC_NONE     = 0,     /* empty adspec */
        RAPI_ADSPEC_IS       = 1      /* Int-serv format */
} rapi_adspec_form_t;

/*
 *      RAPI Adspec
 */

typedef struct {
        u_int32_t            len;
        rapi_adspec_form_t   form;
        union {
                IS_adspec_t       IS_Adspec;       /* Int-serv */
        }               specbody_u;
}   rapi_adspec_t;

/*****************************************************************************/

/*
 *    Policy Formats
 */

typedef struct {
    char        name[1];

} rapi_policy_name_string_t;

typedef enum {
    RAPI_POLICY_TYPE_NAMESTRING   =  1
} rapi_policy_form_t;

typedef enum {
    RAPI_POLICYF_INTEGRITY        =  0x00000001,
    RAPI_POLICYF_GLOBAL           =  0x00000002,
    RAPI_POLICYF_RESP             =  0x00000004
} rapi_policy_flags_t;


/*
 *    RAPI Policy
 */

typedef struct {
    u_int32_t               len;            /* actual length in bytes */
    rapi_policy_form_t      form;           /* policy data format */
    rapi_policy_flags_t     flags;
    union {
        rapi_policy_name_string_t    namestring;

    } pol_u;

} rapi_policy_t;


/*****************************************************************************/

/*
 *    RAPI Object Header
 */

typedef struct {
    u_int32_t                len;        /* actual length in bytes     */
    int                      form;       /* object format              */
#define RAPI_EMPTY_OTYPE     0           /* Format = 0 => empty object */

} rapi_hdr_t;


/*****************************************************************************/

/*
 *      Reservation style ids
 */
typedef enum {
        RAPI_RSTYLE_WILDCARD = 1,       /* STYLE_ID_WF */
        RAPI_RSTYLE_FIXED = 2,          /* STYLE_ID_FF */
        RAPI_RSTYLE_SE = 3              /* STYLE_ID_SE */
} rapi_styleid_t;

/*
 *      Reservation style extension
 *
 *      This structure is currently undefined; in the future, it will allow
 *      passing style-specific parameters for possible new styles.
 */
typedef void rapi_stylex_t;

/*
 *      Registration flags
 */
#define RAPI_DIR_SEND           0x80    /* Send */
#define RAPI_DIR_RECV           0x40    /* Receive */
#define RAPI_REQ_CONFIRM        0x20    /* Request confirmation */

/*****************************************************************************/

/*
 *   RAPI client handle
 */

typedef unsigned int rapi_sid_t;
#define NULL_SID        0       /* Error return from rapi_session */


/*****************************************************************************/

/*
 *   RAPI User callback function
 */

typedef enum {
        RAPI_PATH_EVENT   = 1,
        RAPI_RESV_EVENT   = 2,
        RAPI_PATH_ERROR   = 3,
        RAPI_RESV_ERROR   = 4,
        RAPI_RESV_CONFIRM = 5,
        RAPI_PATH_STATUS  = 9,
        RAPI_RESV_STATUS  = 10
} rapi_eventinfo_t;


typedef int  ((*rapi_event_rtn_t) (
        rapi_sid_t,              /* Which sid generated event    */
        rapi_eventinfo_t,        /* Event type                   */
        rapi_styleid_t,          /* Style ID                     */
        int,                     /* Error code                   */
        int,                     /* Error value                  */
        struct sockaddr_in,      /* Error node address           */
        u_char,                  /* Error flags                  */
#define RAPI_ERRF_InPlace  0x01  /*   Left resv in place        */
#define RAPI_ERRF_NotGuilty 0x02 /*   This rcvr not guilty      */

        int,                     /* Number of filter specs/sender*/
                                 /*      templates in list       */
        rapi_filter_t *,         /* Filter spec/sender templ list*/
        int,                     /* Number of flowspecs/Tspecs   */
        rapi_flowspec_t *,       /* Flowspec/Tspec list          */
        int,                     /* Number of adspecs            */
        rapi_adspec_t *,         /* Adspec list                  */
        void *                   /* Client supplied arg          */
));


/*****************************************************************************/

/*
 *      RAPI External Functions
 */
rapi_sid_t rapi_session(
        struct sockaddr_in *,   /* Session: (Dst addr, port), net order */
        int,                    /* Protocol id                          */
        int,                    /* Flags - not defined                  */
        rapi_event_rtn_t,       /* Address of callback rtn - optional   */
        void *,                 /* App argument to callback - optional  */
        int *                   /* error return code                    */
);

int rapi_sender(
        rapi_sid_t,             /* Session ID                              */
        int,                    /* Flags - not defined                     */
        struct sockaddr_in *,   /* Local host: (Src addr, port), net order */
        rapi_filter_t *,        /* Sender template - not supported         */
        rapi_tspec_t *,         /* Sender Tspec                            */
        rapi_adspec_t *,        /* Sender Adspec                           */
        rapi_policy_t *,        /* Sender policy data - not supported      */
        int                     /* TTL of multicast data                   */
);

int rapi_reserve(
        rapi_sid_t,             /* Session ID                      */
        int,                    /* Flags - not defined             */
        struct sockaddr_in *,   /* Recv host addr - optional       */
        rapi_styleid_t,         /* Style ID                        */
        rapi_stylex_t *,        /* Style extension - not supported */
        rapi_policy_t *,        /* Receiver policy - not supported */
        int,                    /* # of filter specs               */
        rapi_filter_t *,        /* List of filter specs            */
        int,                    /* # of flowspecs                  */
        rapi_flowspec_t *       /* List of flowspecs               */
);

int rapi_release(rapi_sid_t     /* Session ID */);

int rapi_getfd(rapi_sid_t       /* Session ID */);

int rapi_dispatch(void);

int rapi_version(void);

int rapi_set_refresh(rapi_sid_t,/* Session ID */
                     int        /* time interval */);

int rapi_get_refresh(rapi_sid_t,/* Session ID */
                     int *      /* address of time interval */);

/*      Formatting routines
 */
void    rapi_fmt_flowspec(rapi_flowspec_t *, char *, int);
void    rapi_fmt_tspec(rapi_tspec_t *, char *, int);
void    rapi_fmt_adspec(rapi_adspec_t *, char *, int);
void    rapi_fmt_filtspec(rapi_filter_t *, char *, int);


/*****************************************************************************/

/*
 *      RAPI Error Codes
 */
#define RAPI_ERR_OK             0       /* No error                     */
#define RAPI_ERR_INVAL          1       /* Invalid parameter            */
#define RAPI_ERR_MAXSESS        2       /* Too many sessions            */
#define RAPI_ERR_BADSID         3       /* Sid out of legal range.      */
#define RAPI_ERR_N_FFS          4       /* Wrong n_filter/n_flow for style*/
#define RAPI_ERR_BADSTYLE       5       /* Illegal reservation style    */
#define RAPI_ERR_SYSCALL        6       /* Some sort of syscall err; see errno*/
#define RAPI_ERR_OVERFLOW       7       /* Parm list overflow           */
#define RAPI_ERR_MEMFULL        8       /* Not enough memory            */
#define RAPI_ERR_NORSVP         9       /* Daemon doesn't respond/exist.*/
#define RAPI_ERR_OBJTYPE        10      /* Object type error            */
#define RAPI_ERR_OBJLEN         11      /* Object length error          */
#define RAPI_ERR_NOTSPEC        12      /* No sender tspec in rapi_sendr*/
#define RAPI_ERR_INTSERV        13      /* Intserv format error         */
/*                              14       * UNUSED                       */

/*  The following occur only asynchronously, as the error value when
 *      error code = RSVP_Err_API_ERROR
 */
#define RAPI_ERR_BADSEND        14      /* Sender addr not my interface */
#define RAPI_ERR_BADRECV        15      /* Recv addr not my interface */

#define RAPI_ERR_UNSUPPORTED    254
#define RAPI_ERR_UNKNOWN        255


#endif /* H_RSVP_RAPI */
